#include "controls.h"

enum {UP_DOWN,LEFT_RIGHT};

// command structure - used to pass commands to the server
// int code - command code
// int par1 - par4 are parameters for the command

struct Command
{
  int code;
  int par1;
  int par2;
  int par3;
  int par4;
};

struct NormalVector
{
  float a;
  float b;
  float c;
};  

//##################################################
// class that describes local (eg - on the same computer as the
// running server) client

class LocalClient
{
  int perspective;
      
      
  static const int tex_size = 64;
  
  //all below are sizes in TILES and determine
  //lower right corner
  
  // base size of the patch in tiles along x axis
  int vec_x;
  
  // base size of the patch in tiles along z axis
  int vec_z;
  
  // position of the displayed patch (starting point
  // lower left corner) along x axis
  int pos_x;
  
  // position of the displayed patch (starting point
  // lower left corner) along z axis
  int pos_z;
  
  // total size of the patch along x
  int tot_vec_x;
  
  // total size of the patch along z
  int tot_vec_z;
  
  // total position of the patch along x
  int tot_pos_x;
  
  // total position of the patch along z
  int tot_pos_z;
  
  // table for tile vertices data
 
  //angle of rotation of the view
  //in degrees
  double angle;
  
  //current time for the frame
  int CurrentMs;
  
  //absolute time
  double AbsoluteTime;
  
  //limit of time + bonus
  int ClientLimit;
  
  // how many ms for a single client frame
  int ClientTime;

  // keyboard - keystates;
  Keyboard keyboard;
  
  // pointer to map data;
  tile **Map;
  
  // pointer to verices data
  short **Heights;
  
  // helper table for normals
  NormalVector **Normals;
  
  // size of the map along x
  int size_x;
  
  // size of the map along z
  int size_z;
  
  //tables
  
  char *Texture;
  
  float *ColorTable;
  float *NormalTable;
  float *VertexTable;
  float *TextureTable;
  
  // float values for all corner
  
  float vector1[3];
  float vector2[3];
  
  float normal[3];
  
  bool showgrid;
  bool showtris;
    
  public:

  // table of commands
  Command commands[5];

//###########
//# methods #
//###########

  // constructor
  LocalClient(int MinimumClientTime,tile **MapData,
              short **HeightData,int vecx,int vecz,
              int posx, int posz,int sizez,int sizex);
  
  // destructor
  ~LocalClient(void);
  
  // will draw the contents on the screen. Takes
  // ms it has to do its magic as a parameter
  // returns if it managed to draw or not
  // vec_x,vec_z - vector (now many tiles to draw - vertically
  // and horizontally
  // pos_x,pos_z - position from which to draw
  bool Draw();
  
  // determine keyboard/mouse state
  void TestControls(void);
  
  // determine what to do (commands, view changes etc);
  void MakeCommands();
  
  //checks if the remaining time needed to draw elapsed
  bool CheckEnd (void);
  
  //starts the time counting for the single frame
  void StartClientClock(int Bonus);
  
  //set command
  void SetCommands(int code,int par1,int par2,int par3,int par4);
  
  //tessalize terrain - create triangles etc.
  // void TriangulateTerrain(int vec_x,int vec_z,int pos_x,int pos_z);
  
  //draw terrain
  void DrawTerrain();
  
  //create vertex/normal/color arrays
  void CreateArrays();
  
  //find the normal vector
  void FindNormal(void);
  
  //bilinear filtering of normals
  void FilterNormals (NormalVector vec1,NormalVector vec2,NormalVector vec3,NormalVector vec4,NormalVector vec5);
  
  //trilinear filtering of normals ;o)
  void TriFilterNormals (float *vec1,float *vec2,float *vec3,float *vec4,float *vec5,float *vec6,float *vec7,float *vec8,float *vec9);
  
  //limit colors
  float LC(float col1,float col2);
};
