#include "map.h"
#include <iostream>
#include <SDL/SDL.h>
#include <stdlib.h>
#include <math.h>
#include <time.h>

using namespace std;

time_t t;

GameMap::GameMap(int sizex,int sizez)
{
  // so we create the map of size_x, size_y
  size_x = sizex;
  size_z = sizez;
  
  size_x < 8?size_x = 8:size_x = size_x;
  size_z < 8?size_z = 8:size_z = size_z;


  amplitude = 0.42f;
  octaves = 5;
  edge = 2.7;

  Heights = new short*[size_x+1];

  for (int a = 0;a < size_x+1;a++)
    {
      Heights[a] = new short[size_z+1];
    }    
  RandomMap(); 

  // now create the matrix of tiles
  Map = new tile*[size_x];
  for (int a = 0;a < size_x;a++)
    {
      Map[a] = new tile[size_z];
    }    
    // and zero them
  for (int a = 0;a < size_x;a++)
    for (int b = 0;b < size_z;b++)
      {
        Map[a][b].content = NULL;
        Map[a][b].Type = 0;
        Map[a][b].OwnerNorth = 0;
        Map[a][b].OwnerSouth = 0;
        Map[a][b].OwnerEast = 0;
        Map[a][b].OwnerWest = 0;
      }
     //and we're done
  

};

GameMap::~GameMap()
{
  //clear the memory after the map we have done

  //obviously we'll have to clear the content tiles
  //once they will be used
  //but for now it is not necessary
  for (int a = 0;a < size_x;a++)
    {
      delete[]Map[a];
    }
  delete Map;
  
  for (int a = 0;a < size_x+1;a++)
    {
      delete[]Heights[a];
    }
  delete Heights;
  
};

// MAP GENERATOR

void GameMap::RandomMap()
{
  cout << "New terrain in progress\n";
  cout << "Steps: " << octaves << "\n";
  cout << "Amplitude: " << amplitude << "\n";
  cout << "Add Height: " << edge << "\n";
  
  for (int x = 0;x < size_x+1;x++)
    for (int z = 0;z < size_z+1;z++)
      {
        Heights[x][z] = 0;
      }
  
  // prepare the grid
  float **maingrid; //main grid to store heightmap
                    //and noise

  float **helpgrid; //intermediate grid = used for example
                    //for bilinear filtering
                    
  float **storegrid; //array which will store final data
                     //from here we will write the data
                     //to heightmaps array
                     

  maingrid = new float*[size_x+1];

  for (int a = 0;a< (size_x+1);a++)
    {
      maingrid[a] = new float[size_z+1];
    }
    
  storegrid = new float*[size_x+1];

  for (int a = 0;a< (size_x+1);a++)
    {
      storegrid[a] = new float[size_z+1];
    }    
  //now - make the things happen - perlin noise creation

  srand(unsigned(time(&t)) + SDL_GetTicks());

  cout << "Seeding";

  //create noise
    for (int x = 0;x < size_x+1;x++)
      {
        if ((!(x % int(size_x/2))) && (x > 1) && (x < size_x)) cout << "50%";
          else
            if (!(x % int(size_x/25))) cout << "|";
        for (int z = 0;z < size_z+1;z++)
          {
            maingrid[x][z] = (PerlinRand((x*z)+(rand()%100)));
            storegrid[x][z] = maingrid[x][z];
          }
      }    
  cout << "100%\n";

  // now interpolate
  for (int oct = 0;oct < octaves;oct++)
    {
 
    cout << "Step: " << oct << "\n" << "Making half table: ";

    // now cut a piece of maingrid and create intermediate
    // table to add to our project :)

      helpgrid = new float*[size_x+1];

      for (int a = 0;a< (size_x+1);a++)
        {
          helpgrid[a] = new float[size_z+1];

          for (int x = 0;x < size_z+1;x++)
            {
              helpgrid[a][x] = 0;
            }  
        }
    cout << "done. Interpolating:";
        //copy/add code here
        
        for (int x = 0;x < ((size_x+1)/2); x++)
          for (int z = 0;z < ((size_z+1)/2);z ++)
            {
              helpgrid[x*2][z*2] = maingrid[x][z];
            };
        // interpolate x-axis
    cout << " 25%";
        for (int z = 0;z < size_z+1;z+=2)
          for (int x = 1; x < size_x;x+=2)
            {
              helpgrid[x][z] = (helpgrid[x-1][z]+helpgrid[x+1][z])/2.0;
            }  

        for (int x = 0; x< size_x+1;x+=2)
          for (int z = 1;z < size_z;z+=2)
            {
              helpgrid[x][z] = (helpgrid[x][z-1]+helpgrid[x][z+1])/2.0;
            }

        for (int x = 1; x< size_x;x+=2)
          for (int z = 1;z < size_z;z+=2)
            {
              helpgrid[x][z] = (helpgrid[x+1][z-1]+helpgrid[x+1][z+1]+
                                 helpgrid[x-1][z-1]+helpgrid[x-1][z+1])/4.0;
            }
    cout << " 50%";            
        for (int z = 0; z < size_z+1; z++)
          {
            helpgrid[size_x][z] = helpgrid[size_x-1][z];
          }  

        for (int x = 0; x < size_x+1; x++)
          {
            helpgrid[x][size_z] = helpgrid[x][size_z-1];
          }  


        cout << " 75%";
        for (int a = 0;a< (size_x+1);a++)
            {
              delete[]maingrid[a];
            };
            delete[]maingrid;
        
        maingrid = helpgrid;
        helpgrid = NULL;
       
        //end of it here


        for (int x = 0;x < size_x+1;x++)
          for (int z = 0;z < size_z+1;z++)
            {
              storegrid[x][z] += maingrid[x][z]*pow(2.0,oct);
            }
    cout << " 100%.\n";
    }



  cout << "Bilinear Filtering:\n";

//bilinear filtering

  for (int u=0; u < 3;u++)
    {
    for (int x = 0;x <size_x+1;x++)
      for (int z = 0;z <size_z+1;z++)
        {
          maingrid[x][z] = storegrid[x][z];
        }

    cout << "20% ";
    helpgrid = new float*[size_x+1];
      for (int a = 0;a< (size_x+1);a++)
        {
          helpgrid[a] = new float[size_z+1];
        }

// body
    cout << "40% ";    
    for (int x = 1;x < size_x;x++)
      for (int y = 1;y < size_z;y++)
        {
          helpgrid[x][y] = (maingrid[x][y]+maingrid[x+1][y-1]+maingrid[x+1][y+1]+
                           maingrid[x-1][y-1]+maingrid[x-1][y+1])/5;
        }  

// edges
    cout << "60% ";  
    for (int x = 1;x < size_x;x++)
        {
          helpgrid[x][0] = (maingrid[x][0]*2+maingrid[x-1][0]+maingrid[x+1][0]+maingrid[x][1])/5;
          helpgrid[x][size_z] = (maingrid[x][size_z]*2+maingrid[x-1][size_z]+maingrid[x+1][size_z]+maingrid[x][size_z-1])/5;
        }
      
    for (int z = 1;z < size_z;z++)
        {
          helpgrid[0][z] = (maingrid[0][z]*2+maingrid[0][z-1]+maingrid[0][z+1]+maingrid[1][z])/5;
          helpgrid[size_x][z] = (maingrid[size_x][z]*2+maingrid[size_x][z-1]+maingrid[size_x][z+1]+maingrid[size_x-1][z])/5;
        }

//corners

    helpgrid[0][0] = (maingrid[0][0]*2 + maingrid[1][1] + maingrid[1][0] + maingrid[0][1])/5;
    helpgrid[size_x][0] = (maingrid[size_x][0]*2 + maingrid[size_x-1][0] + maingrid[size_x-1][1] + maingrid[size_x][1])/5;
    helpgrid[0][size_z] = (maingrid[0][size_z]*2 + maingrid[0][size_z-1] + maingrid[1][size_z-1] + maingrid[1][size_z])/5;
    helpgrid[size_x][size_z] = (maingrid[size_x][size_z]*2 + maingrid[size_x-1][size_z-1] + maingrid[size_x-1][size_z] + maingrid[size_x][size_z-1])/5;
    
//end of bilinear filtering 
    cout << "80% ";
    for (int a = 0;a< (size_x+1);a++)
      {
        delete[]maingrid[a];
      }
      delete[]maingrid;
     
      maingrid = helpgrid;
      helpgrid = NULL;
       

  for (int x = 0;x <size_x+1;x++)
    for (int z = 0;z <size_z+1;z++)
      {
        storegrid[x][z] = maingrid[x][z];
      }
      cout << "100%.\n";  
    }  
  cout << "Postprocessing: ";
  //add some to the heightfield to rise the terrain
  // arctan filter

  for (int x = 0;x <size_x+1;x++)
    for (int z = 0;z <size_z+1;z++)
      {
        maingrid[x][z] = storegrid[x][z];
      }
  for (int x = 0;x < size_x+1;x++)
    {
    
    if ((!(x % int(size_x/2))) && (x > 1) && (x < size_x)) cout << "50%";
      else
      if (!(x % int(size_x/25))) cout << "|";

    
    for (int z = 0;z < size_z+1;z++)
      {
        maingrid[x][z] = atan(maingrid[x][z]/20.0)*20.0*amplitude+edge;

        if (maingrid[x][z] < 0)
          {
            maingrid[x][z] = 0;
          }  
      }
    }          

  for (int x = 0;x < size_x+1;x++)
    for (int z = 0;z < size_z+1;z++)
      {
        storegrid[x][z] = maingrid[x][z];
      };

  cout << "100%.\n";  

  //delete the uncecesarry matrix

  for (int a = 0;a < (size_x+1);a++)
    {
      delete[]maingrid[a];
    };
  delete[]maingrid;

  //write data from store grid to heights

  for (int x = 0;x < size_x+1;x++)
    for (int z = 0;z < size_z+1;z++)
      {
        Heights[x][z] = short(storegrid[x][z]*160.0);
      }  

  
  for (int a = 0;a< (size_x+1);a++)
    {
      delete[]storegrid[a];
    };
  delete[]storegrid;
};

// perlin pseudo-random number generatior
long double GameMap::PerlinRand(int seed)
  {
    long double result;
    result = pow(float(seed<<13),seed);
    return ( 1.0 - ( (seed * (seed * seed * 15731 + 789221) + 1376312589) & 0x7fffffff) / 1073741824.0);
  }
