//for now a content class is defined here. Later it will be moved
//to seperate *.h

class Content // class from which all content (houses, trees etc) is derived
{
};

// general note on geometry
// 1 gl unit equals 16 meters
// height is expressed in 10 cm increments
// so 1/160 is height 1
// in display 160 equals 100% slope

struct tile
{
	//   u###########r
	//   ##         ##
	//   # #   N   # #
	//   #  #     #  #
  //   #   #   #   #
  //   #    # #    #
  //   # W   #   E #
  //   #    # #    #
  //   #   #   #   #
  //   #  #     #  #
  //   # #   S   # #
  //   ##         ##
  //   l##########lo
  //                 ___
  //                |\
  //                | \
  //                |  \
  //                    \
  //                     \
  //                      \
  //                default projection
  //
  //   u - uper corner
  //   r - right corner
  //   l - left corner
  //   lo - lower corner
  //
  //   N - north
  //   E - east
  //   S - south
  //   W - west
  
	//type of the tile (is it grass, desert, semigrass etc - used
	//to determine 
	unsigned short int Type;
	
	//pointer to the content of this tile - it may be trees, houses
	// roads etc
	Content *content;
	
	//indexes of owners of the tile triangles
  // eg 1 = company 1
  // eg 2 = company 2
  // eg 3 = city 1
  // etc
  
	char OwnerNorth;
	char OwnerEast;
	char OwnerSouth;
	char OwnerWest;
};

//##################################################
//class holding the map geometry

class GameMap
{
    public:

    // size of the map along x axis (south to north)
    int size_x;

    //size of the map along z axis (east to west)
    int size_z;

    //perlin terrain generator data
    float amplitude; //persistance
    float octaves; // how many steps
    float edge; //what is the limit of the edges
    

    // pointer to the map tile data
    tile **Map;
    
    // pointer to map geometry
    short **Heights;

//###########
//# methods #
//###########

    // constructor that makes a map
    GameMap(int sizex,int sizey);
    
    //destructor that kills the map
    ~GameMap();
    
    //creation of a random landscape
    void RandomMap(void);
    
    //pseudo-psedurandom - uses INT as input
    //and returns a precomputed output in <-1.0,1.0> range
    long double PerlinRand(int seed);
    
    //interpolation function - using cosine func as
    //a basis
    float PerinCosInterpolate(float a, float b, float x);
    
};
    
